﻿using System.Windows;
using System.Windows.Interactivity;
using Microsoft.Expression.Interactivity;

namespace VisualStateDemo.Behaviors
{
    public class BindVisualStateBehaviorHandler : Behavior<FrameworkElement>
    {
        //
        // Dependency property "StateName" that you can bind in Blend.
        // Bind this to the enumeration that controls the visual state.
        //
        public static DependencyProperty StateNameProperty = DependencyProperty.Register(
            "StateName",
            typeof(string),
            typeof(BindVisualStateBehaviorHandler),
            new PropertyMetadata(StateNamePropertyChanged));

        public string StateName
        {
            get { return (string)GetValue(StateNameProperty); }
            set { SetValue(StateNameProperty, value); }
        }

        //
        // When the StateName property changes, switch to the
        // new visual state, and play transition animations.
        //
        private static void StateNamePropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs args)
        {
            ((BindVisualStateBehaviorHandler)obj).UpdateVisualState(
                (string)args.NewValue,
                useTransitions: true);
        }

        //
        // When the behavior is first attached, go to the visual state,
        // but don't play any animations.
        //
        protected override void OnAttached()
        {
            UpdateVisualState(
                StateName,
                useTransitions: false);
            base.OnAttached();
        }

        private void UpdateVisualState(string visualState, bool useTransitions)
        {
            if (AssociatedObject != null)
            {
                FrameworkElement stateTarget;
                if (VisualStateUtilities.TryFindNearestStatefulControl(
                    AssociatedObject, out stateTarget))
                {
                    VisualStateUtilities.GoToState(
                        stateTarget,
                        visualState,
                        useTransitions);
                }
            }
        }
    }
}