﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using VisualStateDemo.State.VisualStates;

namespace VisualStateDemo.State
{
    public class StateHandler : INotifyPropertyChanged
    {

        private readonly Dictionary<ScreenStates, List<ScreenStates>> _screenStateRules;

        public event PropertyChangedEventHandler PropertyChanged;
        public SeasonStates SeasonState { get; private set; }
        public ScaleStates ScaleState { get; private set; }
        public ScreenStates ScreenState { get; private set; }


        public StateHandler()
        {
            _screenStateRules = GetScreenStateRules();
        }

      
        public void SetSeasonState(SeasonStates state)
        {
            if (SeasonState == state)
                throw new Exception("Not allowed SeasonState transition from " + SeasonState + " to " + state);
            SeasonState = state;
        }

        public void SetScaleState(ScaleStates state)
        {
            if (ScaleState == state)
                throw new Exception("Not allowed ScaleState transition from " + ScaleState + " to " + state);
            ScaleState = state;
        }

       
        public void SetScreenState(ScreenStates state)
        {
            if (!_screenStateRules[ScreenState].Contains(state))
                throw new Exception("Not allowed ScreenState transition from " + ScreenState + " to " + state);
           ScreenState = state;
        }


        private Dictionary<ScreenStates,List<ScreenStates>> GetScreenStateRules()
        {
            var rules = new Dictionary<ScreenStates, List<ScreenStates>>
            {
                {
                    ScreenStates.Unknown,
                    new List<ScreenStates> {ScreenStates.Welcome}
                },
                {
                    ScreenStates.Welcome,
                    new List<ScreenStates> {ScreenStates.Start}
                },
                {
                    ScreenStates.Start,
                    new List<ScreenStates> {ScreenStates.Welcome,ScreenStates.Simulation}
                },
                {
                    ScreenStates.Simulation,
                    new List<ScreenStates> {ScreenStates.Done}
                },
                {
                    ScreenStates.Done,
                    new List<ScreenStates> {ScreenStates.Start}
                },
            };
            return rules;
        }

    }
}